@@ -81,6 +81,7 @@ NULL
 #' @param shape.dp Shape parameter under a \emph{c ~ G(shape.dp, 1)} prior on the concentration parameter of the DP (prior
 #'	on the set of random effects parameters, \emph{b_1, ..., b_n ~ DP(c,G_0)}
 #'	where \code{n} is the total number of subjects.
+#' @param rate.dp Rate parameter under a \emph{c ~ G(shape.dp, rate.dp)} prior on the concentration parameter of the DP.
 #' @param plot.out A boolean variable indicating whether user wants to return plots with output results.  Defaults to \code{TRUE}.
 #' @param option A character vector of length equal to the total number of multiple membership terms that supplies the prior formulation choice
 #'	for each term.  The elements of \code{option} are confined to choose from among \code{c("mmcar","mmi","mmigrp","mmdp")}.
@@ -143,7 +144,7 @@ NULL
 #' @export dpgrowmult 
 #' @S3method dpgrowmult default
 dpgrowmult			<- function(y, subject, trt, time, n.random, n.fix_degree, formula, random.only, data, Omega, group, subj.aff, W.subj.aff, n.iter, 
-					n.burn, n.thin, strength.mm, shape.dp, plot.out, option, ulabs)
+					n.burn, n.thin, strength.mm, shape.dp, rate.dp, plot.out, option, ulabs)
 					UseMethod("dpgrowmult")
 
 ################################################
@@ -151,7 +152,7 @@ dpgrowmult			<- function(y, subject, trt, time, n.random, n.fix_degree, formula,
 ################################################
 dpgrowmult.default		<- function(y = NULL, subject, trt = NULL, time = NULL, n.random = NULL, n.fix_degree = NULL, formula = NULL, 
 						random.only = FALSE, data = NULL, Omega = NULL, group = NULL, subj.aff, W.subj.aff, 
-					 	n.iter, n.burn, n.thin = 1, strength.mm = 0.35, shape.dp = 1, plot.out = TRUE, option, ulabs = NULL)
+					 	n.iter, n.burn, n.thin = 1, strength.mm = 0.35, shape.dp = 1, rate.dp = 1, plot.out = TRUE, option, ulabs = NULL)
 { ## start function dpgrowmult.default
 
   ############################
@@ -376,6 +377,7 @@ dpgrowmult.default		<- function(y = NULL, subject, trt = NULL, time = NULL, n.ra
   start		<- 1
   out		<- relabel(label.input = subject, start)
   subject	<- out$label.new
+  o		<- order(subject) ## use later to place X, Z, map.subject, map.trt in contiguous order of subject
   subjecti.u	<- out$labeli.u
   map.subject	<- out$dat.label ## colnames = c("label.input","label.new"), in case format
 
@@ -462,10 +464,20 @@ dpgrowmult.default		<- function(y = NULL, subject, trt = NULL, time = NULL, n.ra
   X.n   	<- out$X.n
   Z		<- out$Z
   Z.n		<- out$Z.n
-  Z.c  	<- out$Z.c
-  if( !is.null(out$y) ) y <- out$y  ## over-writes possible duplicative input of y by user (since must be in formula).
+  Z.c  		<- out$Z.c
+  if( !is.null(out$y) ) 
+  {
+	y 	<- out$y  ## over-writes possible duplicative input of y by user (since must be in formula).
+  }else{ ## out$y is null, so user separately entered
+	y	<- y[o] ## re-order y by subject to ensure subject is in contiguous order
+  }
   y	= as.matrix(y,Ncase,1)
 
+  ## reorder remaining objects to subject (in contiguous fashion) where entries indexed by case
+  subject		<- subject[o]
+  map.subject		<- map.subject[o,]
+  map.trt		<- map.trt[o,]
+
   ## capture number of fixed effects
   Nfixed		= ncol(X)
   Nrandom		= ncol(Z)
@@ -519,7 +531,7 @@ dpgrowmult.default		<- function(y = NULL, subject, trt = NULL, time = NULL, n.ra
   }
 
   shape.dp	= 4
-  res 		= mmmultPost(y, X, Z, W.case, Mmats, Omega, omega.plus, ngs, subject, typet, n.iter, n.burn, n.thin, shape.dp, strength.mm)
+  res 		= mmmultPost(y, X, Z, W.case, Mmats, Omega, omega.plus, ngs, subject, typet, n.iter, n.burn, n.thin, shape.dp, rate.dp, strength.mm)
   stopifnot(length(res) == 19)
   stopifnot(ncol(res$Tauu) == nty)
   stopifnot(is.list(res$U))
@@ -570,7 +582,7 @@ dpgrowmult.default		<- function(y = NULL, subject, trt = NULL, time = NULL, n.ra
 	gc.plot		= growthCurve(y.case = y, B = res$B, Alpha = res$Alpha, Beta = res$Beta, U = res$U, aff.clients = subj.aff, W.subj = W.subj, X.n = X.n, Z.n = Z.n, 
 				trt.case = trt, trt.lab = trti.u, subject.case = subject, subject.lab = subjecti.u, T = T, min.T = min.T, max.T = max.T, n.thin = n.thin, 
 				n.waves = n.waves, time.case = time, n.fix_degree = n.fix_degree, Nrandom = n.random)
-			## memo: if have nuisance covariates, need input of Nrandom = n.random to construct time-based random effects since Q > Nrandom
+			## memo: if have nuisance covariates, need input of Nrandom to construct time-based random effects since Q > Nrandom
    }
 
  } ## end conditional statement on creating growth curves
@@ -675,18 +687,19 @@ dpgrowmult.default		<- function(y = NULL, subject, trt = NULL, time = NULL, n.ra
 #' @param nthin The step increment of MCMC samples to return
 #' @param shapealph The shape parameter for the \eqn{\Gamma} prior on the DP concentration parameter.  
 #'	The rate parameter is set of \code{1}.
+#' @param ratebeta The rate parameter for the \eqn{\Gamma} prior on the DP concentration parameter. Default value is \code{1}.
 #' @param strength.mm The shape and rate parameters for the \eqn{\Gamma} prior on the CAR precision parameter, \eqn{\tau_{\gamma}}
 #' @return res A list object containing MCMC runs for all model parameters.
 #' @seealso \code{\link{dpgrowmm}}
 #' @author Terrance Savitsky \email{tds151@@gmail.com}
 #' @note Intended as an internal function for \code{\link{dpgrowmult}}
-mmmultPost = function (y, X, Z, Wcases, Mmats, Omegas, omegapluses, ngs, subjects, typet, niter, nburn, nthin, shapealph, strength.mm) {
+mmmultPost = function (y, X, Z, Wcases, Mmats, Omegas, omegapluses, ngs, subjects, typet, niter, nburn, nthin, shapealph, ratebeta, strength.mm) {
     stopifnot(nrow(X) == nrow(Z))
     stopifnot(nrow(Wcases[[1]]) == nrow(Z))
     stopifnot(length(y) == nrow(X))
     stopifnot(length(omegapluses) == length(Omegas))
     stopifnot(length(Wcases) == length(typet))
-    res <- .Call("mmmult", y, X, Z, Wcases, Mmats, Omegas, omegapluses, ngs, subjects, typet, niter, nburn, nthin, shapealph, strength.mm, package = "growcurves")
+    res <- .Call("mmmult", y, X, Z, Wcases, Mmats, Omegas, omegapluses, ngs, subjects, typet, niter, nburn, nthin, shapealph, ratebeta, strength.mm, package = "growcurves")
 }
 
 ####################################
